/* OpenProcessing Tweak of *@*http://www.openprocessing.org/sketch/63228*@* */
/* !do not delete the line above, required for linking your tweak if you re-upload */
import processing.opengl.*;

import ddf.minim.*;
import ddf.minim.signals.*;
import ddf.minim.analysis.*;
import ddf.minim.effects.*;

import toxi.math.conversion.*;
import toxi.geom.*;
import toxi.math.*;
import toxi.geom.mesh2d.*;
import toxi.util.datatypes.*;
import toxi.util.events.*;
import toxi.geom.mesh.subdiv.*;
import toxi.geom.mesh.*;
import toxi.math.waves.*;
import toxi.util.*;
import toxi.math.noise.*;

import toxi.physics2d.constraints.*;
import toxi.physics2d.behaviors.*;
import toxi.physics2d.*;


//audio variables
Minim minim;
AudioPlayer song;
FFT fft;
float currentBand;
final String trackName = "jouineau_sport.wav";

//effects
HighPassSP highpass;
LowPassSP lowpass;
float baseHighPassFreq = 1; //Hz
float baseLowPassFreq = 22050; //Hz
GainControl gainController;

//graphical setting
final color backgrnd = color(255);

//Spectrum bar variable
List<SpectrumBar> bars;
color barColor;
color mainBarColor = color(18,64,171,255);
color reflectedBarColor = color(18,64,171,150);

//UI controls
SpringControl[] controls;
color UIControlsBackground = color(255);

VerletPhysics2D vp;
Vec2D center;
float angle, angleIncr;
PFont myFont;
String letters = "go~dR go~dR@vij l@ pei d@ me gRa~ paRa~ go~dR@vij la fRa~s";
int counter = 0;
int TailleString;


void setup() {
  size(800, 600, OPENGL);
  
  // maFonte = loadFont("Verdana-48.vlw");
  
    myFont = createFont("Impact", 48);
  textFont(myFont);
 
  
    angleIncr=0;
      center = new Vec2D(width/4, height/2);
  vp = new VerletPhysics2D();
  vp.addBehavior(new GravityBehavior(new Vec2D(0, 0.001)));

  TailleString = letters.length();
println(TailleString);

  
  //init sound
  minim = new Minim(this);
  song = minim.loadFile(trackName, 1024);
  
  //setup UI controls
  controls = new SpringControl[2];
  controls[0] = new SpringControl( width/4, 50, 0.75, 8.0, 0.5, "Frequencies filter");
  controls[1] = new SpringControl( 3*width/4, 50, 0.75, 8.0, 0.5, "Gain");
  
  //init FFT object
  fft = new FFT(song.bufferSize(), song.sampleRate());
  fft.linAverages(75);
  fft.window(FFT.HAMMING);
  
  //init bast spectrum bars
  bars = new ArrayList<SpectrumBar>();
  
  //init filter
  lowpass = new LowPassSP(baseLowPassFreq, 44100);
  highpass = new HighPassSP(baseHighPassFreq, 44100);
  
  //setup gain coltroller
  gainController = new GainControl();
  
  //add effectr to the song
  song.addEffect(highpass);
  song.addEffect(lowpass);
  song.addEffect(gainController);
  
  //start play
  song.play();
  
}

void draw() {
  background(0);
  
 
  
  //compute FFT
  fft.forward(song.mix);
  //...and show each component
  for(int i = 0; i < fft.avgSize(); i++) {
    // get magnitude of fft at band i
    currentBand = fft.getBand(i);
    
    //setup the past bar color
    colorMode(HSB,360);
    int barHue = int(90-min(currentBand*2,90));
    barColor = color(barHue, 360, 360);
    colorMode(RGB);
    
    // Draw a bar for FFT Band i as well as a reflection of the bar
    pushMatrix();
    translate(i*width/fft.avgSize(), height, -width/10);
    stroke(mainBarColor);
    strokeWeight(4);
    line(0, 0, 0, 0, -4*currentBand, 0);
    stroke(reflectedBarColor);
    strokeWeight(4);
    line(0, 0, 0, -2*currentBand, 2*currentBand, currentBand/4);
    popMatrix();
    
    // Make a spot if the magnitude of the band is big enough
    if((currentBand > 1)) {
      bars.add(new SpectrumBar(i*width/fft.avgSize(), height-int(4*currentBand), -width/10, barColor));
    }
  }
  
  // udate the fresh bars and remove the oldest
  for(int i=0; i<bars.size(); i++){
    SpectrumBar bar = bars.get(i);
    if(bar.dead()){
      bars.remove(bar);
    } else {
      bar.update();
    }
  }
  
  // update value of frequency UI control
  //updateFrequencyControl();
  //update value of gain
  //updateGainControl();
  fill(0, 30);
    rect(0, 0, width, height);
    pushMatrix();
    translate(center.x, center.y);
    rotate(radians(angle));
    angle = (angle+angleIncr)%360;
    if (frameCount%2 == 0)
    {
      vp.addParticle(new Particle(0, 0));
    }
    int nbParticles = vp.particles.size();
    for (int i = nbParticles-1; i >=0;--i)
    {
      Particle particle =  (Particle) vp.particles.get(i);
      if (particle.dead())
      {
        vp.removeParticle(particle);
      }
      else
      {
        particle.display();
          
      }
    }
    popMatrix();
    vp.update();
    //println(nbParticles + " " + angleIncr + " " + frameRate);
    lights();
  
} 


void stop() {
  song.close();
  minim.stop();
  super.stop();
}



