public class SpringControl
{ 
  //original positions
  float xpos, ypos;
  // current positions
  float tempxpos, tempypos; 
  
  //handle size
  int wSize = 20; 
  int hSize = 40;
  
  //maximum absolute value
  float maxOffset = 100.0;
  
  boolean over = false; 
  boolean move = false; 

  // Spring simulation constants 
  private float mass;       // Mass 
  private float k = 0.2;    // Spring constant 
  private float damp;       // Damping 

  // Spring simulation variables 
  private float vel = 0.0;   // X Velocity 
  private float accel = 0;    // Acceleration 
  private float force = 0;    // Force 

  String title;  //spring title
  PFont font;
  
  //element's color
  color guideColor = color(18,64,171);
  color textColor = color(18,64,171);
  color elasticColor = color(255,184,115);
  color handleNotOverColor = color(255,127,00);
  color handleOverColor = color(246,00,24);
  
  // Constructor
  SpringControl(float x, float y, float d, float m, 
         float k_in, String title) 
  { 
    xpos = tempxpos = x; 
    ypos = tempypos = y;
    damp = d; 
    mass = m; 
    k = k_in;
    font = loadFont("DejaVuSans-48.vlw");
    textFont(font,17);
    this.title=title;
  } 

  void update() 
  { 
    if (move) { 
      tempxpos = constrain(mouseX, xpos-maxOffset+1, xpos+maxOffset-1);    
    }  else {
      force = -k * (tempxpos - xpos);    // f=-ky 
      accel = force / mass;              // Set the acceleration, f=ma == a=f/m 
      vel = damp * (vel + accel);        // Set the velocity 
      tempxpos = tempxpos + vel;         // Updated position 
    }
    
    if (over() || move ) { 
      over = true; 
    } else { 
      over = false; 
    } 
  } 
  
  // Test to see if mouse is over this spring
  boolean over() {
    float disX = abs(tempxpos - mouseX);
    float disY = abs(ypos - mouseY);
    if ((disX < wSize/2) && (disY < hSize/2)) {
      return true;
    } else {
      return false;
    }
  }

  void display() 
  { 
    //display title
    fill(textColor);
    textAlign(CENTER);
    text(title, xpos, ypos-hSize/2-5);
    
    //display hint
    textAlign(RIGHT);
    text("+", xpos+maxOffset, ypos-hSize/2-5);
    textAlign(LEFT);
    text("-", xpos-maxOffset, ypos-hSize/2-5);
    
    //display guide
    strokeWeight(1);
    stroke(guideColor);
    line(xpos-maxOffset,ypos,xpos+maxOffset,ypos);
    noStroke();
    
    // Draw elastic
    fill(elasticColor);
    float tension_height = hSize/maxOffset;
    rectMode(CENTER);
    rect(int((xpos+tempxpos)/2), ypos, abs(xpos-tempxpos), hSize-abs(xpos-tempxpos)*tension_height);
    
    //display handle
    if (over) { 
      fill(handleOverColor); 
    } else { 
      fill(handleNotOverColor); 
    } 
    rect(tempxpos, tempypos, wSize, hSize);
  } 

  void pressed() { 
    if (over) { 
      move = true; 
    } else { 
      move = false; 
    }  
  } 

  void released() { 
    move = false; 
  } 
  
  float getValue() {
    return (tempxpos-xpos);
  }
} 
